/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor;

import static org.eclipse.ui.forms.widgets.ExpandableComposite.EXPANDED;
import static org.eclipse.ui.forms.widgets.ExpandableComposite.TITLE_BAR;
import static org.eclipse.ui.forms.widgets.ExpandableComposite.TWISTIE;
import static org.eclipse.ui.forms.widgets.Section.DESCRIPTION;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.widgets.ColumnLayout;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;
import org.fortiss.tooling.kernel.ui.extension.base.EditorBase;

/**
 * Basic editor based on forms.
 * <p>
 * This editor disables the library view.
 * 
 * @author ratiu
 */
public abstract class FormsEditorBase<T extends EObject> extends EditorBase<T> {

	/** Our toolkit. */
	protected FormToolkit toolkit;

	/** The main form. */
	protected ScrolledForm form;

	/** Grid data factory used. */
	protected final GridDataFactory gridDataFactory = GridDataFactory.fillDefaults();

	/** {@inheritDoc} */
	@Override
	public final void createPartControl(Composite parent) {
		toolkit = new FormToolkit(parent.getDisplay());

		form = toolkit.createScrolledForm(parent);
		form.setText("Overview");
		toolkit.decorateFormHeading(form.getForm());

		form.getBody().setLayout(new ColumnLayout());

		setupScrolledForm(form);

		createSections();
	}

	/** {@inheritDoc} */
	@Override
	public void setFocus() {
		form.setFocus();
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		toolkit.dispose();

		super.dispose();
	}

	/**
	 * Setup the form for this editor. Subclass can override this method to
	 * setup the form before {@link #createSections()}, e.g. add actions to the
	 * tool bar.
	 * 
	 * @param form
	 *            the scrolled form.
	 */
	protected void setupScrolledForm(ScrolledForm form) {
		// do nothing by default
	}

	/** Creates the sections for this editor. */
	protected abstract void createSections();

	/**
	 * Creates a new section and returns the client which is used to add
	 * controls to. The client is assigned a 3 column grid layout, but this can
	 * be changed.
	 */
	protected Composite createNewSection(String title, String description) {
		Section section = toolkit.createSection(form.getForm().getBody(),
				EXPANDED | TWISTIE | TITLE_BAR | DESCRIPTION);
		section.setText(title);
		section.setDescription(description);
		Composite client = toolkit.createComposite(section);
		section.setClient(client);

		toolkit.paintBordersFor(client);
		client.setLayout(new GridLayout(3, false));

		return client;
	}
}
